#include "fonctions.h"

/* fct puissance */
double puiss(double x,int n) {
	if (n<=0) return 1;
	if (n==1) return x;
	else {
		if (n&1) return x*puiss(x,n/2)*puiss(x,n/2);
		else return puiss(x,n/2)*puiss(x,n/2);
	}
}

/* Homma-Saltelli: (loi uniforme sur [-pi,pi]) */
std::vector<double> hs(std::vector<double>& X, long S) {
	double aux1,aux2;
	std::vector<double> res(S);
	for (long i=S-1; i>=0; i--) {
		aux1=sin(X[i]); 
		aux2=sin(X[S+i]); 
		res[i]=aux1+7*aux2*aux2+0.1*puiss(X[2*S+i],4)*aux1;
	}
	return res;
}

/* modele a deux dimensions de la fonction precedente */
std::vector<double> hs2(std::vector<double>& X, long S) {
	double aux1,aux2;
	std::vector<double> res(2*S);
	for (long i=S-1; i>=0; i--) {
		aux1=sin(X[i]);
		aux2=sin(X[S+i]);
		res[i]=aux1+7*aux2*aux2+0.1*puiss(X[2*S+i],4)*aux1;
		res[S+i]=aux2+7*aux1*aux1+0.1*puiss(X[2*S+i],4)*aux2;
	}
	return res;
}

/* fonction g de Sobol: (produit des phi_i(t)) */
std::vector<double> g(std::vector<double>& X, long S) {	
	int* a=(int*)malloc(8*sizeof(int));
	a[0]=0;
	a[1]=0;
	for (int i=2; i<8; i++) a[i]=3;
	std::vector<double> res(S);
	for (long i=S-1; i>=0; i--) {
		res[i]=1;
		for (int j=7; j>=0; j--) res[i]*=(2*X[j*S+i]+a[j])/(1+a[j]);
	}
	return res;
}

/* fonction chameau (cf. Scheidt 2006) */
std::vector<double> cham(std::vector<double>& X, long S) {
	std::vector<double> res(S);
	for (long i=S-1; i>=0; i--) { 
		res[i]=1.9*puiss(X[i],4)+(1.0/3)*puiss(X[i],6)+X[i]*X[i+S]-4*X[i+S]*X[i+S]+4*puiss(X[i+S],4);
	}
	return res;
}

/* fonction chameau decale (cf. Scheidt 2006) */
std::vector<double> chamdec(std::vector<double>& X, long S) {
	std::vector<double> res(S);
	for (long i=S-1; i>=0; i--) {
		res[i]=-89.1*puiss(X[i],4)+243*puiss(X[i],6)+9*X[i]*X[i+S]-36*X[i+S]*X[i+S]+81*puiss(X[i+S],4);
	}
	return res;
}

/* fonction non reguliere 2 (cf. Scheidt 2006) */
std::vector<double> scheidt2(std::vector<double>& X, long S) {
	std::vector<double> res(S);
	for (long i=S-1; i>=0; i--) {
		res[i]=0.2*exp(X[i]-3)+2.2*fabs(X[i+S])+1.3*puiss(X[i+S],6)-2*X[i+S]*X[i+S]-0.5*puiss(X[i+S],4)-0.5*puiss(X[i],4)+2.5*X[i]*X[i]+0.7*X[i]*X[i]*X[i]+3.0/(puiss(8*X[i]-2,2)+puiss(5*X[i+S]-3,2)+1)+sin(5*X[i])*cos(3*X[i]*X[i]);
	}
	return res;
}

/* fonction dosimetre_cuivre (cf. Beretz et al. 2003) */
std::vector<double> dosiCu(std::vector<double>& X, long S) {
	std::vector<double> res(S);
	for (long i=S-1; i>=0; i--) { 
	  res[i]=5.8044e11+9.9320e10*X[i]-3.6102e11*X[i+S]+8.2874e10*puiss(X[i+S],2)-8.5334e9*puiss(X[i+S],3)+3.4654e8*puiss(X[i+S],4)-9.9719e9*X[i]*X[i+S];
	}
	return res;
}

/* fonction dosimetre_U238 (cf. Beretz et al. 2003) */
std::vector<double> dosiU(std::vector<double>& X, long S) {
	std::vector<double> res(S);
	for (long i=S-1; i>=0; i--) { 
	  res[i]=-5.25428+2.35964e-3*X[i]-19.1466*X[i+6*S]+9.04741e-3*X[i+2*S]+5.19686e-3*X[i+3*S]+4.20388*X[i+4*S]+0.440531e-10*X[i+5*S]+10.7146*X[i+S]-4.43213e-3*X[i]*X[i+6*S]-3.46927e-3*X[i]*X[i+4*S]-1.81083e-2*X[i+3*S]*X[i+S]-8.87977e-3*X[i+2*S]*X[i+S]-8.05995e-14*X[i+2*S]*X[i+5*S]-0.705689e-10*X[i+5*S]*X[i+S]-2.15082*X[i+S]*X[i+S];
	    }
	return res;
}



