#include "indEntrop.h"

void entrop(std::vector<double> f(std::vector<double>&,long),int dim_x,int dim_y,long S,int n,int R,
	int* lois,double* paramlois,bool* logx,bool* logy,bool* indices1,bool* indices2,
	bool* rel, bool deterministe,bool affichage) { 

/*----------------------------------------------------------------------------------------------
*Estimation des indices de sensibilite basee sur l'entropie
*-----------------------------------------------------------------------------------------------
*Arguments :
*		f : pointeur sur fonction
*		dim_x : nombre de parametres d'entree dans le modele
*		dim_y : nombre de parametres en sortie
*		S : nombre de simulations
*		n : nombre de sous-intervalles de variations des parametres
*		R : nombre de repetitions du programme
*		lois : vecteur contenant les types de distribution de proba
*			pour chaque Vi
*			0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull
*			4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapezoidale
*			Par defaut, on prend la loi uniforme
*		paramlois : tableau contenant les parametres de chaque loi
*				pour chaque Vi (range par colonne) :
*				(min,max) pour uniforme, (moy,ecart-type) pour loi normale,
*				(moy du log, ecart-type du log) pour loi lognormale,
*				(forme,echelle) pour loi de Weibull, (lambda) pour loi
*				exponentielle, (n, lambda) pour loi gamma
*				Par defaut, on a la loi uniforme avec (min,max) calcules a
*				partir des parametres d'entree x
*		logx : vecteur correspondant aux parametres d'entree transformes
*			en log lors de l'ajustement du modele
*		logy : vecteur correspondant aux parametres de sortie transformes
*			en log lors de l'ajustement du modele
*		rel : TRUE si normalisation par la somme des indices
*		indices1 et 2: indique les indices (simples et doubles) devant etre
*					calcules; par defaut, seulement les indices simples
*		deterministe : TRUE pour un echantillonage ameliore, non aleatoire
*		affichage : TRUE pour indiquer les r�ultats interm�iaires

*-----------------------------------------------------------------------------------------------
*Sortie : les indices eta a la fin du fichier "res-ENTROP.out" (fichier non ecrase)
*-----------------------------------------------------------------------------------------------*/

int i,ip,ipp,iip,tmpi;
long j,aux=puiss(n+1,dim_y);
double tmp,sum;
int *estimationsX=NULL,*estimationsXX=NULL;
std::ofstream res_Entrop("res-ENTROP.out",std::ios_base::app);

/* initialisation des tableaux contenant les indices */
bool Si=false,Sij=false;
for (i=dim_x-1; i>=0; i--)
	if (indices1[i]) {
		Si=true;
		break;
	}
double *mS1=NULL,*minS1=NULL,*maxS1=NULL,*sigS1=NULL,*HK=NULL;
if (Si) {
	HK=(double*)calloc(dim_x*R,sizeof(double));
	mS1=(double*)malloc(dim_x*sizeof(double));
	minS1=(double*)malloc(dim_x*sizeof(double));
	maxS1=(double*)malloc(dim_x*sizeof(double));
	sigS1=(double*)malloc(dim_x*sizeof(double));
} 
for (i=0; i<dim_x-1; i++) 
	for (ip=i+1; ip<dim_x; ip++)
		if (indices2[i*dim_x+ip]) {
			Sij=true;
			break;
		}
double *mS2=NULL,*minS2=NULL,*maxS2=NULL,*sigS2=NULL,*HK2=NULL;
if (Sij) {
	HK2=(double*)calloc(dim_x*dim_x*R,sizeof(double));
	mS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	minS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	maxS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	sigS2=(double*)malloc(dim_x*dim_x*sizeof(double));
} 

/* autres initialisations */
double* pasX=(double*)malloc(dim_x*sizeof(double));
double* infX=(double*)malloc(dim_x*sizeof(double));
double* pasY=(double*)malloc(dim_y*sizeof(double));
double* infY=(double*)malloc(dim_y*sizeof(double));
long *p=(long*)malloc(dim_y*sizeof(long));
for (i=dim_y-1; i>=0; i--) p[i]=puiss(n+1,i);
if (Si) estimationsX=(int*)malloc((n+1)*sizeof(int));
if (Sij) estimationsXX=(int*)malloc((n+1)*(n+1)*sizeof(int));

/*-------
*Boucle |
*-------*/

/* on repete le programme R fois */
for (int z=0; z<R; z++) { 

if (affichage) printf("passage %i dans la boucle\n",z+1);

/* Tirage de l'echantillon */
std::vector<double> X=sampling(dim_x,S,lois,paramlois,z,deterministe); 
for (i=dim_x-1; i>=0; i--) {
	if (logx[i]) 		
		for (j=S-1; j>=0; j--) X[i*S+j]=log(X[i*S+j]);
}

/* precalculs: decoupages des intervalles de variation des echantillons + calcul des bornes inferieures */
for (i=dim_x-1; i>=0; i--) {
	if (lois[i]==0) {
		pasX[i]=(paramlois[i*4+1]-paramlois[i*4])/n;
		infX[i]=paramlois[i*4];
	}
	else if (lois[i]==5) {
		pasX[i]=(double)1/n;
		infX[i]=0;
	}
	else if (lois[i]==6) {
		pasX[i]=(paramlois[i*4+2]-paramlois[i*4])/n;
		infX[i]=paramlois[i*4];	
	}     
	else if (lois[i]==7) {
		pasX[i]=(paramlois[i*4+3]-paramlois[i*4])/n;
		infX[i]=paramlois[i*4];
	}
	else {
		tmp=min(X,i,S);
		pasX[i]=(max(X,i,S)-tmp)/n;	
		infX[i]=tmp;
	}
} 

/*-----------------------
*Calcul de la reponse Y |
*----------------------*/

std::vector<double> Y=f(X,S);
for (i=dim_y-1; i>=0; i--) {
	if (logy[i])
		for(j=S-1; j>=0; j--) Y[i*S+j]=exp(Y[i*S+j]);
}	

/* precalculs: decoupages des intervalles de variation des Y[,i] */
for (i=dim_y-1; i>=0; i--) {
	tmp=min(Y,i,S);
	pasY[i]=(max(Y,i,S)-tmp)/n;	
	infY[i]=tmp;
}

/* precalcul: un vecteur a dim_y dimensions est vu comme un nombre en base n+1, < (n+1)^aux, stock�dans casesY
* estimationsY : tableau de repartition spatiale des Y[,i]  */
std::vector<long> casesY(S);
std::vector<int> estimationsY(aux);
for (j=S-1; j>=0; j--) {
	for (i=dim_y-1; i>=0; i--) casesY[j]+=p[i]*(long)floor((Y[i*S+j]-infY[i])/pasY[i]);
	estimationsY[casesY[j]]++;
}

/*-----------------------------------
*Calcul de H0==H(Y) (si necessaire) |
*-----------------------------------*/

double H0=0;
if (!rel) {
	for (j=aux-2; j>=0; j--)
		if (estimationsY[j]>0) {
			tmp=(double)estimationsY[j]/S;
			H0-=tmp*log(tmp);
		}
	if (affichage) printf("entropie: %f\n",H0);
}

/*--------------
*Calcul des Hi |
*--------------*/

std::vector<int> casesX(dim_x*S);

if (Si) {

	std::vector<int> estimationsXY((n+1)*aux);

	for(i=dim_x-1; i>=0; i--)
		if (indices1[i]) { 
			/* reinitialisations a zero */
			for (ip=n; ip>=0; ip--) {
				estimationsX[ip]=0;
				for (j=aux-1; j>=0; j--) estimationsXY[ip*aux+j]=0;
			}

			/* precalcul: tableau de repartition spatiale de X[,i], puis de (X[,i],Y) */
			for (j=S-1; j>=0; j--) { 
				tmpi=(int)floor((X[i*S+j]-infX[i])/pasX[i]);
				estimationsX[tmpi]++;
				casesX[i*S+j]=tmpi; 
				estimationsXY[tmpi*aux+casesY[j]]++; 
			}

			for (ip=n-1; ip>=0; ip--)
				for (j=aux-2; j>=0; j--)
					if (estimationsXY[ip*aux+j]>0) {
						tmp=(double)estimationsXY[ip*aux+j]/S;
						HK[i*R+z]+=tmp*log(tmp/(((double)estimationsX[ip]/S)*((double)estimationsY[j]/S)));
					}
		}
}

else {
	/* calcul de casesX (meme principe que casesY) */
	for (i=dim_x-1; i>=0; i--)
	for (j=S-1; j>=0; j--) casesX[i*S+j]=(int)floor((X[i*S+j]-infX[i])/pasX[i]);
}

/*----------------
*Calcul des Hij |
*----------------*/

if (Sij) {

	std::vector<int> estimationsXXY((n+1)*(n+1)*aux);

	for(i=0; i<dim_x-1; i++) 
		for (ip=i+1; ip<dim_x; ip++)
			if (indices2[i*dim_x+ip]) {
				/* reinitialisations a zero */
				for (ipp=n; ipp>=0; ipp--) 
					for (iip=n; iip>=0; iip--) {
						estimationsXX[ipp*(n+1)+iip]=0;
						for (j=aux-1; j>=0; j--) estimationsXXY[ipp*aux*(n+1)+iip*aux+j]=0;
					}
						
				/* precalcul: tableau de repartition spatiale de (X[,i],X[,j],Y), et (X[,i],X[,j]):*/
				for (j=S-1; j>=0; j--) {
					estimationsXX[casesX[i*S+j]*(n+1)+casesX[ip*S+j]]++;
					estimationsXXY[casesX[i*S+j]*(n+1)*aux+casesX[ip*S+j]*aux+casesY[j]]++;
				}
	
				for (ipp=n-1; ipp>=0; ipp--)
					for (iip=n-1; iip>=0; iip--)
						for (j=aux-2; j>=0; j--) 
							if (estimationsXXY[ipp*(n+1)*aux+iip*aux+j]>0) {
								tmp=(double)estimationsXXY[ipp*(n+1)*aux+iip*aux+j]/S;
								HK2[i*dim_x*R+ip*R+z]+=tmp*log(tmp/(((double)estimationsXX[ipp*(n+1)+iip]/S)*((double)estimationsY[j]/S)));
							}
			}	
}

/*-------------------
*Calcul des indices |
*-------------------*/

/* relatifs */
if (Si) 
	if (rel[0]) {
		sum=0;
		for (i=dim_x-1; i>=0; i--) sum+=HK[i*R+z];
		for (i=dim_x-1; i>=0; i--) HK[i*R+z]/=sum;
	}
	else for (i=dim_x-1; i>=0; i--) HK[i*R+z]/=H0;
		
if (Sij) {
	if (rel[1]) {
		sum=0;
		for (i=0; i<dim_x-1; i++)
			for (ip=i+1; ip<dim_x; ip++) sum+=HK2[i*dim_x*R+ip*R+z];
		for (i=0; i<dim_x-1; i++)
			for (ip=i+1; ip<dim_x; ip++) HK2[i*dim_x*R+ip*R+z]/=sum;
	}
	else 
		for (i=0; i<dim_x-1; i++)
			for (ip=i+1; ip<dim_x; ip++) HK2[i*dim_x*R+ip*R+z]/=H0;
}

if (Si && affichage) {
	printf("indices1:\n");
	for (i=0; i<dim_x; i++) printf("%f ",HK[i*R+z]);
	printf("\n");
}

if (Sij && affichage) {
	printf("indices2:\n");
	for (i=0; i<dim_x-1; i++) {
		for (ip=0; ip<=i; ip++) printf("   **   ");
		for (ip=i+1; ip<dim_x; ip++) printf("%f",HK2[i*dim_x*R+ip*R+z]);
		printf("\n");
	}
}

/*------------------
*Fin de la boucle |
*------------------*/

}

/*--------------------------------------------
*Calcul des moyennes, min, max et ecart-types |
*--------------------------------------------*/

/* initialisations des tableaux */
if (Si) {
	mS1=(double*)malloc(dim_x*sizeof(double));
	minS1=(double*)malloc(dim_x*sizeof(double));
	maxS1=(double*)malloc(dim_x*sizeof(double));
	sigS1=(double*)malloc(dim_x*sizeof(double));
} 

if (Sij) {
	mS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	minS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	maxS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	sigS2=(double*)malloc(dim_x*dim_x*sizeof(double));
} 

if (Si) {

	for(i=dim_x-1; i>=0; i--) {
		mS1[i]=mean(HK,i,(long)R);
		minS1[i]=min(HK,i,(long)R);
		maxS1[i]=max(HK,i,(long)R);
		sigS1[i]=sig(HK,i,(long)R);
	}

}

if (Sij) {

	for(i=0; i<dim_x-1; i++)
		for(ip=i+1; ip<dim_x; ip++) {
			mS2[i*dim_x+ip]=mean(HK2,i,ip,dim_x,(long)R);
			minS2[i*dim_x+ip]=min(HK2,i,ip,dim_x,(long)R);
			maxS2[i*dim_x+ip]=max(HK2,i,ip,dim_x,(long)R);
			sigS2[i*dim_x+ip]=sig(HK2,i,ip,dim_x,(long)R);
		}

}

/*-----------------------
* Affichage dans fichier |
*-----------------------*/

res_Entrop << "--------------------------------------------------" << std::endl << std::endl;

if (Si) { 

	res_Entrop << "------------------" << std::endl;
	res_Entrop << "| Les indices Si  |" << std::endl;
	res_Entrop << "------------------" << std::endl;

	for(i=0; i<dim_x; i++)
		if (indices1[i]) {
			res_Entrop << " S" << i+1 << " : ";
			if (R>1) res_Entrop << "Moy=" << mS1[i] << " Min=" << minS1[i]
					<< " Max=" << maxS1[i] << " E-t=" << sigS1[i] << std::endl;
		        else res_Entrop << mS1[i] << std::endl;
	        }

	free(mS1);
	free(minS1);
	free(maxS1);
	free(sigS1);
	free(HK);

}

if (Sij) {

	res_Entrop << "-------------------" << std::endl;
	res_Entrop << "| Les indices Sij  |" << std::endl;
	res_Entrop << "-------------------" << std::endl;

	for(i=0; i<dim_x-1; i++)
		for(ip=i+1; ip<dim_x; ip++)
			if (indices2[i*dim_x+ip]) {
				res_Entrop << " S" << i+1 << ip+1 << " : ";
				if (R>1) res_Entrop << "Moy=" << mS2[i*dim_x+ip] << " Min=" <<
						minS2[i*dim_x+ip]<< " Max=" << maxS2[i*dim_x+ip] << 
						" E-t=" << sigS2[i*dim_x+ip] << std::endl;
				else res_Entrop << mS2[i*dim_x+ip] << std::endl;
			}

	free(mS2);
	free(minS2);
	free(maxS2);
	free(sigS2);
	free(HK2);

}

/* liberations memoire */
if (Si) {
	free(estimationsX);
	free(estimationsXX);
}
free(p);
free(infY);
free(pasY);
free(infX);
free(pasX);

return;

}
