#include "indMix.h"

void mix(std::vector<double> f(std::vector<double>&,long),int dim_x,long S,int n,int R,
            int* lois,double* paramlois,bool* logx,bool logy,bool* indices1,
            bool* indices2,bool deterministe,bool affichage) {

/*----------------------------------------------------------------------------------------------
*Estimation des indices de sensibilite mixtes (Y de dimension 1)
*-----------------------------------------------------------------------------------------------
*Arguments :
*	f : pointeur sur fonction
*	dim_x : nombre de parametres d'entree dans le modele
*	S : nombre de simulations
*	n : nombre de sous-intervalles de variations des parametres
*	R : nombre de repetitions du programme
*	lois : vecteur contenant les types de distribution de proba
*		pour chaque Vi
*		0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull
*		4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapezoidale
*		Par defaut, on prend la loi uniforme
*	paramlois : tableau contenant les parametres de chaque loi
*		pour chaque Vi (range par colonne) :
*		(min,max) pour uniforme, (moy,ecart-type) pour loi normale,
*		(moy du log, ecart-type du log) pour loi lognormale,
*		(forme,echelle) pour loi de Weibull, (lambda) pour loi
*		exponentielle, (n, lambda) pour loi gamma
*		Par defaut, on a la loi uniforme sur [0,1]
*	logx : parametres d'entree transformees en log lors de l'ajustement du modele
*	logy : mettre logy=TRUE si la variable de sortie a ete
*		transformee en log lors de l'ajustement du modele
*	indices1 et 2 : indique les indices (simples et doubles) devant etre
*		calcules; par defaut : tous les indices simples
*	rel : TRUE si normalisation par la somme des indices
*	deterministe : TRUE pour un echantillonage ameliore, non aleatoire
*	affichage : TRUE pour indiquer les r�ultats interm�iaires
*-----------------------------------------------------------------------------------------------
*Sortie : les indices Mi, Mij a la fin du fichier "res-MIX.out" (fichier non ecrase)
*-----------------------------------------------------------------------------------------------*/

int i,ip,ipp,iip,iipp,tmpi;
long j;
double sum,tmp;
std::ofstream res_Mix("res-MIX.out",std::ios_base::app);

int *estimationsX=NULL,*estimationsXX=NULL;

/* initialisation des tableaux contenant les indices */
bool Si=false,Sij=false;
for (i=dim_x-1; i>=0; i--) 
	if (indices1[i]) {
		Si=true;
		break;
	}
double *mS1=NULL,*minS1=NULL,*maxS1=NULL,*sigS1=NULL,*M=NULL;
if (Si) {
	M=(double*)calloc(dim_x*S,sizeof(double));
	mS1=(double*)malloc(dim_x*sizeof(double));
	minS1=(double*)malloc(dim_x*sizeof(double));
	maxS1=(double*)malloc(dim_x*sizeof(double));
	sigS1=(double*)malloc(dim_x*sizeof(double));
}
for (i=0; i<dim_x-1; i++) 
	for (ip=i+1; ip<dim_x; ip++) 
		if (indices2[i*dim_x+ip]) {
			Sij=true;
			break;
	} 
double *mS2=NULL,*minS2=NULL,*maxS2=NULL,*sigS2=NULL,*M2=NULL;
if (Sij) {
	M2=(double*)calloc(dim_x*dim_x*S,sizeof(double));
	mS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	minS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	maxS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	sigS2=(double*)malloc(dim_x*dim_x*sizeof(double));
} 

/* autres initialisations */
double* pasX=(double*)malloc(dim_x*sizeof(double));
double* infX=(double*)malloc(dim_x*sizeof(double));
if (Si) estimationsX=(int*)malloc((n+1)*sizeof(int));
if (Sij) estimationsXX=(int*)malloc((n+1)*(n+1)*sizeof(int));

/*-------
*Boucle |
*-------*/

/* on repete le programme R fois */
for (int z=0; z<R; z++) { 

if (affichage) printf("passage %i dans la boucle\n",z+1);

/* Tirage de l'echantillon pour la variance */
std::vector<double> X1=sampling(dim_x,S,lois,paramlois,z,false);
for (i=dim_x-1; i>=0; i--) { 
	if (logx[i]) 		
		for (j=S-1; j>=0; j--) X1[i*S+j]=log(X1[i*S+j]);
}
std::vector<double> X2=sampling(dim_x,S,lois,paramlois,z+R,false);
for (i=dim_x-1; i>=0; i--) {
	if (logx[i]) 		
		for (j=S-1; j>=0; j--) X2[i*S+j]=log(X2[i*S+j]);
}

/*---------------------------
* Calcul de la moyenne de y |
*---------------------------*/

std::vector<double> Y1=f(X1,S); 
if (logy) for(j=S-1; j>=0; j--) Y1[j]=exp(Y1[j]);
double infY=min(Y1,S),supY=max(Y1,S);
double my=0;
for (j=S-1; j>=0; j--) my+=Y1[j];
my=my/S; 

/*---------------
* Calcul des Vi |
*---------------*/

std::vector<double> Y2;

if (Si) {

	std::vector<double> sav1(S);

	for(i=dim_x-1; i>=0; i--) 
		if (indices1[i]) {
			for (j=S-1; j>=0; j--) {  
				sav1[j]=X2[i*S+j];
				X2[i*S+j]=X1[i*S+j];
			}
			Y2=f(X2,S);
			if (logy)
				for (j=S-1; j>=0; j--) Y2[j]=exp(Y2[j]);
			for (j=S-1; j>=0; j--) M[i*R+z]+=Y1[j]*Y2[j];
			M[i*R+z]=exp(12*fabs(M[i*R+z]/S-my*my)/(pow(supY-infY,2))-1);
			for (j=S-1; j>=0; j--) X2[i*S+j]=sav1[j];
		}
}

/*----------------
* Calcul des Vij |
*----------------*/

if (Sij) {

	std::vector<double> sav2(2*S);

	for (i=0; i<dim_x-1; i++)
		for (ip=i+1; ip<dim_x; ip++) 
			if (indices2[i*dim_x+ip]) {
				for (j=S-1; j>=0; j--) {
					sav2[j]=X2[i*S+j];
					sav2[S+j]=X2[ip*S+j];
					X2[i*S+j]=X1[i*S+j];
					X2[ip*S+j]=X1[ip*S+j];
				}
				Y2=f(X2,S);
				if (logy)
					for (j=S-1; j>=0; j--) Y2[j]=exp(Y2[j]);
				for (j=S-1; j>=0; j--) M2[i*dim_x*R+ip*R+z]+=Y1[j]*Y2[j];
				M2[i*dim_x*R+ip*R+z]=exp(12*fabs(M2[i*dim_x*R+ip*R+z]/S-my*my)/(pow(supY-infY,2))-1);
				for (j=S-1; j>=0; j--) {
					X2[i*S+j]=sav2[j];
					X2[ip*S+j]=sav2[S+j];
				} 
			}
}

/* Tirage de l'echantillon pour l'entropie */
std::vector<double> X=sampling(dim_x,S,lois,paramlois,z,deterministe);
for (i=dim_x-1; i>=0; i--) {
	if (logx[i]) 		
		for (j=S-1; j>=0; j--) X[i*S+j]=log(X[i*S+j]);
}

/*-----------------------
*Calcul de la reponse Y |
*----------------------*/

std::vector<double> Y=f(X,S);
if (logy) for(j=S-1; j>=0; j--) Y[j]=exp(Y[j]);

/*---------------------
*Precalculs sur X et Y |
*----------------------*/

/* bornes sur les echantillons et sur Y + decoupage des intervalles de variation */
for (i=dim_x-1; i>=0; i--) {
	if (lois[i]==0) {
		pasX[i]=(paramlois[i*4+1]-paramlois[i*4])/n;
		infX[i]=paramlois[i*4];
	}
	else if (lois[i]==5) {
		pasX[i]=(double)1/n;
		infX[i]=0;
	}
	else if (lois[i]==6) {
		pasX[i]=(paramlois[i*4+2]-paramlois[i*4])/n;
		infX[i]=paramlois[i*4];
	}
	else if (lois[i]==7) {
		pasX[i]=(paramlois[i*4+3]-paramlois[i*4])/n;
		infX[i]=paramlois[i*4];
	}
	else {
		tmp=min(X,i,S);
		pasX[i]=(max(X,i,S)-tmp)/n;
		infX[i]=tmp;
	}
}
double pasY=(supY-infY)/n;
infY=min(Y,S);
supY=max(Y,S);

/* precalcul: indices de estimationsY */
std::vector<int> casesY(S);
for (j=S-1; j>=0; j--) casesY[j]=(long)floor((Y[j]-infY)/pasY);

/*--------------
*Calcul des Hi |
*--------------*/

std::vector<int> casesX(dim_x*S);

if (Si) {

	std::vector<int> estimationsXY((n+1)*(n+1));

	for(i=dim_x-1; i>=0; i--)
		if (indices1[i]) {
			/* reinitialisations */
			for (ip=n; ip>=0; ip--) {
				estimationsX[j]=0;
				for (ipp=n; ipp>=0; ipp--) estimationsXY[ip*(n+1)+ipp]=0;
			}
		
			/* precalcul: tableau de repartition spatiale de X[,i], puis de (X[,i],Y) */
			for (j=S-1; j>=0; j--) {
				tmpi=(int)floor((X[i*S+j]-infX[i])/pasX[i]);
				estimationsX[tmpi]++;
				casesX[i*S+j]=tmpi;
				estimationsXY[tmpi*(n+1)+casesY[j]]++;
			}

			sum=0;
			for (ip=n-1; ip>=0; ip--)
				for (ipp=n-1; ipp>=0; ipp--)
					if (estimationsXY[ip*(n+1)+ipp]>0) {
						tmp=(double)estimationsXY[ip*(n+1)+ipp]/(pasX[i]*pasY*S); 
						sum+=tmp*log(pasX[i]*S*tmp/estimationsX[ip])*pasX[i]*pasY;
					}
			M[i*R+z]+=pow(1.1,sum);
		}
}

else { 
	/* calcul de casesX */
	for (i=dim_x-1; i>=0; i--)
		for (j=S-1; j>=0; j--) casesX[i*S+j]=(int)floor((X[i*S+j]-infX[i])/pasX[i]);
}

/*----------------
*Calcul des Hij |
*----------------*/

if (Sij) {

	std::vector<int> estimationsXXY((n+1)*(n+1)*(n+1));

	for(i=0; i<dim_x-1; i++) 
		for (ip=i+1; ip<dim_x; ip++)
			if (indices2[i*dim_x+ip]) {
				/* reinitialisations */
				for (ipp=n; ipp>=0; ipp--) {
					for (iip=n; iip>=0; iip--) {
						estimationsXX[ipp*(n+1)+iip]=0;
						for (iipp=n; iipp>=0; iipp--) estimationsXXY[ipp*(n+1)*(n+1)+iip*(n+1)+iipp]=0;
					}
				}
						
				/* precalcul: tableau de repartition spatiale de (X[,i],X[,j],Y), et (X[,i],X[,j]):*/
				for (j=S-1; j>=0; j--) {
					estimationsXX[casesX[i*S+j]*(n+1)+casesX[ip*S+j]]++;
					estimationsXXY[casesX[i*S+j]*(n+1)*(n+1)+casesX[ip*S+j]*(n+1)+casesY[j]]++;
				}
	
				sum=0;
				for (ipp=n-1; ipp>=0; ipp--)
					for (iip=n-1; iip>=0; iip--)
						for (iipp=n-1; iipp>=0; iipp--) 
							if (estimationsXXY[ipp*(n+1)*(n+1)+iip*(n+1)+iipp]>0) {
								tmp=(double)estimationsXXY[ipp*(n+1)*(n+1)+iip*(n+1)+iipp]/(pasX[i]*pasX[ip]*pasY*S);                               
								sum+=tmp*log(pasX[i]*pasX[ip]*S*tmp/estimationsXX[ipp*(n+1)+iip])*pasX[i]*pasX[ip]*pasY;
							}
				M2[i*dim_x*R+ip*R+z]+=pow(1.1,sum);
			}
}

/*------------------ 
*Calcul des indices |
*------------------*/

if (Si) {
	sum=0;
	for (i=dim_x-1; i>=0; i--) sum+=fabs(M[i*R+z]);
	for (i=dim_x-1; i>=0; i--) M[i*R+z]=fabs(M[i*R+z])/sum;
}
if (Sij) {
	sum=0;
	for (i=0; i<dim_x-1; i++)
		for (ip=i+1; ip<dim_x; ip++) sum+=fabs(M2[i*dim_x*R+ip*R+z]);
	for (i=0; i<dim_x-1; i++)
		for (ip=i+1; ip<dim_x; ip++) M2[i*dim_x*R+ip*R+z]=fabs(M2[i*dim_x*R+ip*R+z])/sum;
}

if (Si && affichage) {
	printf("indices1:\n");
	for (i=0; i<dim_x; i++) printf("%f ",M[i*R+z]);
	printf("\n");
}

if (Sij && affichage) {
	printf("indices2:\n");
	for (i=0; i<dim_x-1; i++) {
		for (ip=0; ip<=i; ip++) printf("   **   ");
		for (ip=i+1; ip<dim_x; ip++) printf("%f",M2[i*dim_x*R+ip*R+z]);
		printf("\n");
	}
}

/*------------------
*Fin de la boucle |
*------------------*/

}

/*--------------------------------------------
*Calcul des moyennes, min, max et ecart-types |
*--------------------------------------------*/

if (Si) {
	mS1=(double*)malloc(dim_x*sizeof(double));
	minS1=(double*)malloc(dim_x*sizeof(double));
	maxS1=(double*)malloc(dim_x*sizeof(double));
	sigS1=(double*)malloc(dim_x*sizeof(double));
} 

if (Sij) {
	mS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	minS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	maxS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	sigS2=(double*)malloc(dim_x*dim_x*sizeof(double));
} 

if (Si) {

	for(i=dim_x-1; i>=0; i--) {
		mS1[i]=mean(M,i,(long)R);
		minS1[i]=min(M,i,(long)R);
		maxS1[i]=max(M,i,(long)R);
		sigS1[i]=sig(M,i,(long)R);
	}
}

if (Sij) {

	for(i=0; i<dim_x-1; i++)
		for(ip=i+1; ip<dim_x; ip++) {
			mS2[i*dim_x+ip]=mean(M2,i,ip,dim_x,(long)R);
			minS2[i*dim_x+ip]=min(M2,i,ip,dim_x,(long)R);
			maxS2[i*dim_x+ip]=max(M2,i,ip,dim_x,(long)R);
			sigS2[i*dim_x+ip]=sig(M2,i,ip,dim_x,(long)R);
		}
}

/*-----------------------
* Affichage dans fichier |
*-----------------------*/

res_Mix << "--------------------------------------------------" << std::endl << std::endl;

if (Si) {

	res_Mix << "------------------" << std::endl;
	res_Mix << "| Les indices Si  |" << std::endl;
	res_Mix << "------------------" << std::endl;

	for(i=0; i<dim_x; i++)
		if (indices1[i]) {
			res_Mix << " S" << i+1 << " : ";
			if (R>1) res_Mix << "Moy=" << mS1[i] << " Min=" << minS1[i] 
				<< " Max=" << maxS1[i] << " E-t=" << sigS1[i] << std::endl;
			else res_Mix << mS1[i] << std::endl;
		}

	free(mS1);
	free(minS1);
	free(maxS1);
	free(sigS1);
	free(M);
}

if (Sij) {

	res_Mix << "-------------------" << std::endl;
	res_Mix << "| Les indices Sij  |" << std::endl;
	res_Mix << "-------------------" << std::endl;

	for(i=0; i<dim_x-1; i++)
		for(ip=i+1; ip<dim_x; ip++)
			if (indices2[i*dim_x+ip]) {
				res_Mix << " S" << i+1 << ip+1 << " : ";
				if (R>1) res_Mix << "Moy=" << mS2[i*dim_x+ip] << " Min=" <<
					minS2[i*dim_x+ip]<< " Max=" << maxS2[i*dim_x+ip] << 
					" E-t=" << sigS2[i*dim_x+ip] << std::endl;
				else res_Mix << mS2[i*dim_x+ip] << std::endl;
			}
         
	free(mS2);
	free(minS2);
	free(maxS2);
	free(sigS2);
	free(M2);
}

free(infX);
free(pasX);
if (Si) free(estimationsX);
if (Sij) free(estimationsXX);

return;
}
