#include "indSobol.h"

#include <iostream>

void sobol(std::vector<double> f(std::vector<double>&,long),int dim_x,long S,int R,
	int* lois,double* paramlois,bool* logx,bool logy,
	bool* indices1,bool* indices2,bool* indicest,bool* rel,bool affichage) {  
 
/*---------------------------------------------------------------------------
* Fonction pour calculer les indices de sensibilite de Sobol                
*---------------------------------------------------------------------------
*Arguments :
*		f : pointeur sur la fonction
*    	dim_x : nombre de parametres d'entree dans le modele
*		S : nombre de simulations
*		R : nombre de repetitions du programme
*		lois : vecteur contenant les types de distribution de proba
*			pour chaque Vi
*			0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull
*			4=exponentielle
*			Par defaut, on prend la loi uniforme
*		paramlois : tableau contenant les parametres de chaque loi
*				pour chaque Vi (range par colonne) :
*				(min,max) pour uniforme, (moy,ecart-type) pour loi normale,
*				(moy du log, ecart-type du log) pour loi lognormale,
*				(forme,echelle) pour loi de Weibull, (lambda) pour loi
*				exponentielle, (n, lambda) pour loi gamma
*				Par defaut, on a la loi uniforme avec (min,max) calcules a
*				partir des parametres d'entree x
*		logx : vecteur logx=c(TRUE,TRUE,...,TRUE), correspondant aux
*     		parametres d'entree transformees en log lors de
*			l'ajustement du modele
*		logy : mettre logy=TRUE si la variable de sortie a ete
*			transformee en log lors de l'ajustement du modele
*		indices1,2 et t : indique les indices (simples, doubles et totaux) devant etre
*					calcules; par defaut, seulement les indices simples
*		affichage: TRUE pour afficher les r�ultats interm�iaires
*---------------------------------------------------------------------------
*Sortie : les indices Si, Sij et STi a la fin du fichier "res-SOBOL.out"    
*         (fichier non ecrase)                                              
*---------------------------------------------------------------------------*/

int i,ip;
long j;
double sum=0;
std::ofstream res_Sobol("res-SOBOL.out",std::ios_base::app);

/* initialisation des tableaux contenant les indices */
bool Si=false, Sij=false, Sti=false;
for (i=dim_x-1; i>=0; i--) 
	if (indices1[i])  {
		Si=true;
		break;
	}
double *mS1=NULL,*minS1=NULL,*maxS1=NULL,*sigS1=NULL,*S1=NULL;
if (Si) {
	S1=(double*)calloc(dim_x*R,sizeof(double));
	mS1=(double*)malloc(dim_x*sizeof(double));
	minS1=(double*)malloc(dim_x*sizeof(double));
	maxS1=(double*)malloc(dim_x*sizeof(double));
	sigS1=(double*)malloc(dim_x*sizeof(double));
}
for (i=0; i<dim_x-1; i++) 
	for (ip=i+1; ip<dim_x; ip++)
		if (indices2[i*dim_x+ip]) {
			Sij=true;
			break;
		}
double *mS2=NULL,*minS2=NULL,*maxS2=NULL,*sigS2=NULL,*S2=NULL;
if (Sij) {
	S2=(double*)calloc(dim_x*dim_x*R,sizeof(double));
	mS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	minS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	maxS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	sigS2=(double*)malloc(dim_x*dim_x*sizeof(double));
} 
for (i=dim_x-1; i>=0; i--) 
	if (indicest[i])  {
		Sti=true;
		break;
	}
double *mSt=NULL,*minSt=NULL,*maxSt=NULL,*sigSt=NULL,*St=NULL;
if (Sti) {
	St=(double*)calloc(dim_x*R,sizeof(double));
	mSt=(double*)malloc(dim_x*sizeof(double));
	minSt=(double*)malloc(dim_x*sizeof(double));
	maxSt=(double*)malloc(dim_x*sizeof(double));
	sigSt=(double*)malloc(dim_x*sizeof(double));
}

/*--------
* Boucle |
*--------*/

for (int z=0; z<R; z++) { 

if (affichage) printf("passage %i dans la boucle\n",z+1);

/* Tirage de l'echantillon */
std::vector<double> X1=sampling(dim_x,S,lois,paramlois,z,false); 
for (i=dim_x-1; i>=0; i--) { 
	if (logx[i]) 		
		for (j=S-1; j>=0; j--) X1[i*S+j]=log(X1[i*S+j]);
} 
std::vector<double> X2=sampling(dim_x,S,lois,paramlois,z+R,false); 
for (i=dim_x-1; i>=0; i--) {
	if (logx[i]) 		
		for (j=S-1; j>=0; j--) X2[i*S+j]=log(X2[i*S+j]);
}

/*---------------------------
* Calcul de la moyenne de y |
*---------------------------*/

std::vector<double> Y=f(X1,S); 

if (logy) 
	for(j=S-1; j>=0; j--) Y[j]=exp(Y[j]); 
double my=0;
for (j=S-1; j>=0; j--) my+=Y[j];
my=my/S; 

/*-------------
* Calcul de V0 |
*-------------*/

double V0=0;
for (j=S-1; j>=0; j--) V0+=Y[j]*Y[j];
V0=V0/S-my*my;

if (affichage) printf("variance: %f\n",V0);

/*---------------
* Calcul des Vi |
*---------------*/

std::vector<double> Yp;

if (Si) {

	std::vector<double> sav1(S);

	for(i=dim_x-1; i>=0; i--)
		if (indices1[i]) { 
			for (j=S-1; j>=0; j--) {
				sav1[j]=X2[i*S+j];
				X2[i*S+j]=X1[i*S+j];
			}
			Yp=f(X2,S);
			if (logy)
				for (j=S-1; j>=0; j--) Yp[j]=exp(Yp[j]);
			for (j=S-1; j>=0; j--) S1[i*R+z]+=Y[j]*Yp[j];
			S1[i*R+z]=S1[i*R+z]/S-my*my;
			for (j=S-1; j>=0; j--) X2[i*S+j]=sav1[j];
		}

}

/*----------------
* Calcul des Vij |
*----------------*/

if (Sij) {

	std::vector<double> sav2(2*S);
	double Sid=0,Sjd=0;

	for(i=0; i<dim_x-1; i++)
		for(ip=i+1; ip<dim_x; ip++)
			if (indices2[i*dim_x+ip]) {
				// s'il manque des valeurs, il faut les completer
				if (!indices1[i]) {
					for (j=S-1; j>=0; j--) {
						sav2[j]=X2[i*S+j];
						X2[i*S+j]=X1[i*S+j];
					}
					Yp=f(X2,S);
					if (logy)
						for (j=S-1; j>=0; j--) Yp[j]=exp(Yp[j]);
					for (j=S-1; j>=0; j--) Sid+=Y[j]*Yp[j];
					Sid=Sid/S-my*my;
					for (j=S-1; j>=0; j--) X2[i*S+j]=sav2[j];
				}
				else Sid=S1[i*R+z];
				if (!indices1[ip]) {
					for (j=S-1; j>=0; j--) {
						sav2[j]=X2[ip*S+j];
						X2[ip*S+j]=X1[ip*S+j];
					}
					Yp=f(X2,S);
					if (logy)
						for (j=S-1; j>=0; j--) Yp[j]=exp(Yp[j]);
					for (j=S-1; j>=0; j--) Sjd+=Y[j]*Yp[j];
					Sjd=Sjd/S-my*my;
					for (j=S-1; j>=0; j--) X2[ip*S+j]=sav2[j];
	        	}
				else Sjd=S1[ip*R+z];
				// calcul de Sij
				for (j=S-1; j>=0; j--) {
					sav2[j]=X2[i*S+j];
					sav2[S+j]=X2[ip*S+j];
					X2[i*S+j]=X1[i*S+j];
					X2[ip*S+j]=X1[ip*S+j];
				}
				Yp=f(X2,S); 
				if (logy)
					for (j=S-1; j>=0; j--) Yp[j]=exp(Yp[j]);
				for (j=S-1; j>=0; j--) S2[i*dim_x*R+ip*R+z]+=Y[j]*Yp[j];
				S2[i*dim_x*R+ip*R+z]=S2[i*dim_x*R+ip*R+z]/S-Sid-Sjd-my*my;
				for (j=S-1; j>=0; j--) {
					X2[i*S+j]=sav2[j];
					X2[ip*S+j]=sav2[S+j];
				} 
			}
		
}

/*----------------
* Calcul des Vti |
*----------------*/

if (Sti) {  

	std::vector<double> savt(dim_x*S);

	for(i=dim_x-1; i>=0; i--)
		for (j=S-1; j>=0; j--) savt[i*S+j]=X1[i*S+j]; 

	for (i=dim_x-1; i>=0; i--) 
		if (indicest[i]) { 
			for (j=S-1; j>=0; j--) savt[i*S+j]=X2[i*S+j];
			Yp=f(savt,S); 
			if (logy)
				for (j=S-1; j>=0; j--) Yp[j]=exp(Yp[j]);
			for (j=S-1; j>=0; j--) St[i*R+z]+=Y[j]*Yp[j];
			St[i*R+z]=St[i*R+z]/S-my*my; 
			for (j=S-1; j>=0; j--) savt[i*S+j]=X1[i*S+j];
		}

}

/*-----------------------------
* Calcul des indices de Sobol |
*-----------------------------*/

if (Si) 
	if (rel[0]) {
		sum=0;
		for (i=dim_x-1; i>=0; i--) sum+=S1[i*R+z];
		for (i=dim_x-1; i>=0; i--) S1[i*R+z]/=sum;
	}
	else	
		for (i=dim_x-1; i>=0; i--) S1[i*R+z]=S1[i*R+z]/V0;

if (Sij) 
	if (rel[1]) {
		sum=0;
		for (i=0; i<dim_x-1; i++) 
			for (ip=i+1; ip<dim_x; ip++) sum+=S2[i*dim_x*R+ip*R+z];
		for (i=0; i<dim_x-1; i++) 
			for (ip=i+1; ip<dim_x; ip++) S2[i*dim_x*R+ip*R+z]/=sum;
	}
	else
		for (i=dim_x-1; i>=0; i--) 
			for (ip=dim_x-1; ip>=0; ip--) S2[i*dim_x*R+ip*R+z]=S2[i*dim_x*R+ip*R+z]/V0;

if (Sti) {
	sum=0;
	for (i=dim_x-1; i>=0; i--) {
		St[i*R+z]=1-St[i*R+z]/V0;
		sum+=St[i*R+z];
	}
	if (rel[2]) 
		for (i=dim_x-1; i>=0; i--) St[i*R+z]/=sum;
}

if (Si && affichage) {
	printf("indices1:\n");
	for (i=0; i<dim_x; i++) printf("%f ",S1[i*R+z]);
	printf("\n");
}

if (Sij && affichage) {
	printf("indices2:\n");
	for (i=0; i<dim_x-1; i++) {
		for (ip=0; ip<=i; ip++) printf("   **   ");
		for (ip=i+1; ip<dim_x; ip++) printf("%f",S2[i*dim_x*R+ip*R+z]);
		printf("\n");
	}
}

if (Sti && affichage) {
	printf("indicest:\n");
	for (i=0; i<dim_x; i++) printf("%f ",St[i*R+z]);
	printf("\n");
}

/*------------------
* Fin de la boucle |
*------------------*/

}

/*--------------------------------------------
*Calcul des moyennes, min, max et ecart-types |
*--------------------------------------------*/

/* initialisations des tableaux */
if (Si) {
	mS1=(double*)malloc(dim_x*sizeof(double));
	minS1=(double*)malloc(dim_x*sizeof(double));
	maxS1=(double*)malloc(dim_x*sizeof(double));
	sigS1=(double*)malloc(dim_x*sizeof(double));
} 

if (Sij) {
	mS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	minS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	maxS2=(double*)malloc(dim_x*dim_x*sizeof(double));
	sigS2=(double*)malloc(dim_x*dim_x*sizeof(double));
} 

if (Sti) {
	mSt=(double*)malloc(dim_x*sizeof(double));
	minSt=(double*)malloc(dim_x*sizeof(double));
	maxSt=(double*)malloc(dim_x*sizeof(double));
	sigSt=(double*)malloc(dim_x*sizeof(double));
}                 

if (Si) {

	for(i=dim_x-1; i>=0; i--) {
		mS1[i]=mean(S1,i,(long)R);
		minS1[i]=min(S1,i,(long)R);
		maxS1[i]=max(S1,i,(long)R);
		sigS1[i]=sig(S1,i,(long)R);
	}

}

if (Sij) {

	for(i=0; i<dim_x-1; i++)
		for(ip=i+1; ip<dim_x; ip++) {
			mS2[i*dim_x+ip]=mean(S2,i,ip,dim_x,(long)R);
			minS2[i*dim_x+ip]=min(S2,i,ip,dim_x,(long)R);
			maxS2[i*dim_x+ip]=max(S2,i,ip,dim_x,(long)R);
			sigS2[i*dim_x+ip]=sig(S2,i,ip,dim_x,(long)R);
		}	

}

if (Sti) {

	for(i=dim_x-1; i>=0; i--) {
		mSt[i]=mean(St,i,(long)R);
		minSt[i]=min(St,i,(long)R);
		maxSt[i]=max(St,i,(long)R);
		sigSt[i]=sig(St,i,(long)R);
	}

}

/*-----------------------
* Affichage dans fichier |
*-----------------------*/

res_Sobol << "--------------------------------------------------" << std::endl << std::endl;

if (Si) {

	res_Sobol << "------------------" << std::endl;
	res_Sobol << "| Les indices Si  |" << std::endl;
	res_Sobol << "------------------" << std::endl;

	for(i=0; i<dim_x; i++)
		if (indices1[i]) {
			res_Sobol << " S" << i+1 << " : ";
			if (R>1) res_Sobol << "Moy=" << mS1[i] << " Min=" << minS1[i] 
					<< " Max=" << maxS1[i] << " E-t=" << sigS1[i] << std::endl;
			else res_Sobol << mS1[i] << std::endl;
		}

	free(mS1);
	free(minS1);
	free(maxS1);
	free(sigS1);
	free(S1);

}

if (Sij) {

	res_Sobol << "-------------------" << std::endl;
	res_Sobol << "| Les indices Sij  |" << std::endl;
	res_Sobol << "-------------------" << std::endl;

	for(i=0; i<dim_x-1; i++)
		for(ip=i+1; ip<dim_x; ip++)
			if (indices2[i*dim_x+ip]) {
				res_Sobol << " S" << i+1 << ip+1 << " : ";
				if (R>1) res_Sobol << "Moy=" << mS2[i*dim_x+ip] << " Min=" <<
						minS2[i*dim_x+ip]<< " Max=" << maxS2[i*dim_x+ip] << 
						" E-t=" << sigS2[i*dim_x+ip] << std::endl;
				else res_Sobol << mS2[i*dim_x+ip] << std::endl;
			}
         
	free(mS2);
	free(minS2);
	free(maxS2);
	free(sigS2);
	free(S2);

}

if (Sti) {

	res_Sobol << "------------------" << std::endl;
	res_Sobol << "| Les indices Sti |" << std::endl;
	res_Sobol << "------------------" << std::endl;

	for(i=0; i<dim_x; i++)
		if (indicest[i]) {
			res_Sobol << " St" << i+1 << " : ";
			if (R>1) res_Sobol << "Moy=" << mSt[i] << " Min=" << minSt[i] 
					<< " Max=" << maxSt[i] << " E-t=" << sigSt[i] << std::endl;
			else res_Sobol << mSt[i] << std::endl;
		}
    
	free(mSt);
	free(minSt);
	free(maxSt);
	free(sigSt);
	free(S2);
               
}

return;

}

