#include <cstdlib>
#include <fstream>
#include <iostream>
#include <string>
#include "indSobol.h"
#include "indEntrop.h"
#include "indMix.h"
#include "fonctions.h"

using namespace std;

typedef vector<double> (*pf)(vector<double>&,long);
/* transformation d'une chaine de caracteres en fonction  */
pf getFunc(string name) {
	if (name=="hs") return hs;
	else if (name=="hs2") return hs2;
	else if (name=="g") return g;
	else if (name=="ch") return cham;
	else if (name=="chd") return chamdec;
	else if (name=="sch") return scheidt2;
	else if (name=="dosiCu") return dosiCu;
	else if (name=="dosiU") return dosiU;
	else return NULL;
}

int main(int argc, char **argv) { 

/* Programme principal; 3 modes d'utilisations possibles:
             
*** mode le moins interactif:

* argv[1]: nom du fichier contenant les donnees
* argv[2]: s pour variance, e pour entropie, m pour mixte
* argv[3]: nom de la fonction
* argv[4]: nombre de points

*** mode semi-interactif: 

* argv[1]: nom du fichier contenant les donnees
* le reste est demande en ligne de commande

*** mode totalement interactif:
    
* tout est demande en ligne de commande    
    
*/    

int R=0,dim_x=0,dim_y=0,n=0,mem,i,j;
int *lois=NULL;
long S=0;
bool affichage=false,deterministe=false;
bool *indices1=NULL,*indices2=NULL,*indicest=NULL,*logx=NULL,*logy=NULL,*rel=NULL;
char choix;
double *paramlois=NULL;
string line,name,tmp,fichier;
	
/* lecture des parametres dans un fichier, puis appel des fonctions */
if (argc>1) {

	ifstream file(argv[1]);

	if (argc<=2) {
		cout << "Quel programme voulez-vous? s: sobol classique; e: entropie; m: mix entropie/variance" << endl;
		cin >> choix;
	}
	else choix=argv[2][0];

	// on ignore une ligne sur deux, correspondants aux commentaires
	getline(file,line,'\n');
	getline(file,line,'\n');
	// on supprime les espaces dans la ligne si necessaire 
	for (i=0; i<(int)line.size() && line[i]!=' '; i++) {}
	// on lit alors la valeur
	dim_x=atoi((line.substr(0,i)).c_str());

	getline(file,line,'\n');
	getline(file,line,'\n');
	for (i=0; i<(int)line.size() && line[i]!=' '; i++) {}
	dim_y=atoi((line.substr(0,i)).c_str());

	getline(file,line,'\n');
	getline(file,line,'\n');
	int i=0;
	for (; i<(int)line.size() && line[i]!=' '; i++) {}
	n=atoi((line.substr(0,i)).c_str());      

	getline(file,line,'\n');
	getline(file,line,'\n');
	for (i=0; i<(int)line.size() && line[i]!=' '; i++) {}
	R=atoi((line.substr(0,i)).c_str());  

	getline(file,line,'\n');
	getline(file,line,'\n');
	lois=(int*)malloc(dim_x*sizeof(int));
	for (i=0; i<dim_x; i++) lois[i]=(line[i]-'0'); 

	/* attention: remplir toutes les cases, meme si inutile */
	getline(file,line,'\n');  
	getline(file,line,'\n');  
	paramlois=(double*)malloc(4*dim_x*sizeof(double));
	string tmps;
	for (int z=0; z<4; z++) { 
		i=0;
		j=0;
		while (j<dim_x) { 
			mem=i;
			for (; i<(int)line.size() && line[i]!=' '; i++) {}
			tmps=line.substr(mem,i-mem); 
			paramlois[j*4+z]=atof(tmps.c_str()); 
			i++; j++;
			if (i>=(int)line.size() || line[i]=='\n' || line[i]==' ') break;
		} 
		getline(file,line,'\n');
		if (line[0]>='A' && line[0]<='z') break;
	} 

	getline(file,line,'\n'); 
	logx=(bool*)calloc(dim_x,sizeof(bool));
	for (i=0; i<dim_x; i++) 
		if (line[i]!='0') logx[i]=true;

	getline(file,line,'\n'); 
	getline(file,line,'\n');
	logy=(bool*)calloc(dim_y,sizeof(bool));
	for (i=0; i<dim_y; i++) 
		if (line[i]!='0') logy[i]=true;

	getline(file,line,'\n');
	getline(file,line,'\n');
	indices1=(bool*)calloc(dim_x,sizeof(bool));
	for (i=0; i<dim_x; i++) 
		if (line[i]!='0') indices1[i]=true;

	getline(file,line,'\n');
	indices2=(bool*)calloc(dim_x,dim_x*sizeof(bool));
	for (i=0; i<dim_x; i++) {
		getline(file,line,'\n');
		for (j=0; j<dim_x; j++) 
			if (line[j]!='0') indices2[i*dim_x+j]=true;
	}

	getline(file,line,'\n');
	getline(file,line,'\n');
	indicest=(bool*)calloc(dim_x,sizeof(bool));
	for (i=0; i<dim_x; i++) 
		if (line[i]!='0') indicest[i]=true;

	getline(file,line,'\n'); 
	getline(file,line,'\n');
	rel=(bool*)calloc(3,sizeof(bool));
	for (i=0; i<3; i++) 
		if (line[i]!='0') rel[i]=true;

	getline(file,line,'\n');
	getline(file,line,'\n'); 
	if (line[0]!='0') deterministe=true;

	getline(file,line,'\n');
	getline(file,line,'\n');
	if (line[0]!='0') affichage=true;
 
	file.close(); 

	if (argc>2) {

		if (choix=='s') sobol(getFunc(argv[3]),dim_x,atol(argv[4]),R,lois,paramlois,
					logx,logy[0],indices1,indices2,indicest,rel,affichage);

		else if (choix=='e') entrop(getFunc(argv[3]),dim_x,dim_y,atol(argv[4]),n,R,lois,paramlois,
					logx,logy,indices1,indices2,rel,deterministe,affichage);
		
		else if (choix=='m') mix(getFunc(argv[3]),dim_x,atol(argv[4]),n,R,lois,paramlois,
					logx,logy[0],indices1,indices2,deterministe,affichage);
	}

	else {
     
		// completion de la ligne de commande
		cout << "Entrez le nom de la fonction" << endl;
		cin >> name;
		cout << "Entrez le nombre de points dans l'echantillon" << endl;
		cin >> S;     
	
		if (choix=='s') sobol(getFunc(name),dim_x,S,R,lois,paramlois,logx,logy[0],
					indices1,indices2,indicest,rel,affichage);
	
		else if (choix=='e') entrop(getFunc(name),dim_x,dim_y,S,n,R,lois,paramlois,logx,logy,
					indices1,indices2,rel,deterministe,affichage);   
		
		else if (choix=='m') mix(getFunc(name),dim_x,S,n,R,lois,paramlois,logx,logy[0],
					indices1,indices2,deterministe,affichage);
	}     
	return 0;
}

// completion de la ligne de commande
cout << "Quel programme voulez-vous? s: sobol classique; e: entropie; m: mix variance/entropie" << endl;
cin >> choix;
cout << "Entrez le nom de la fonction" << endl;
cin >> name;
cout << "Entrez le nombre de points dans l'echantillon" << endl;
cin >> S;

// remplissage des donnees
cout << "Entrez le nombre de parametres d'entree" << endl;
cin >> dim_x;

if (choix=='e') {

	cout << "Entrez le nombre de dimensions du vecteur sortie" << endl;
	cin >> dim_y;
}

if (choix!='s') {	
	
	cout << "Entrez le nombre de sous-intervalles" << endl;
	cin >> n;
}

else dim_y=1; 

cout << "Entrez le nombre de repetitions de l'algorithme souhaitees" << endl; 
cin >> R;

cout << "Entrez le vecteur des lois" << endl;
cin >> tmp;
lois=(int*)malloc(dim_x*sizeof(int));
for (i=0; i<dim_x; i++) lois[i]=(tmp[i]-'0');

cout << "Entrez la matrice des parametres des lois (# pour indiquer une fin prematuree)" << endl;
/* attention: remplir toutes les cases, meme si inutile */
paramlois=(double*)malloc(4*dim_x*sizeof(double));
for (i=0; i<4; i++) {
	for (j=0; j<dim_x; j++) {
		cin >> tmp; 
		if (tmp[0]=='#') break;
		paramlois[j*4+i]=atof(tmp.c_str());
	}
	if (tmp[0]=='#') break;
}

cout << "Entrez le vecteur des transformations logaritmiques en entree" << endl;
cin >> tmp;
logx=(bool*)calloc(dim_x,sizeof(bool));
for (i=0; i<dim_x; i++) 
	if (tmp[i]!='0') logx[i]=true;

cout << "Entrez le vecteur des transformations logaritmiques en sortie" << endl;
cin >> tmp;
logy=(bool*)calloc(dim_y,sizeof(bool));
for (i=0; i<dim_y; i++) 
	if (tmp[i]!='0') logy[i]=true;

cout << "Entrez le vecteur des indices d'ordre 1 a calculer" << endl;
cin >> tmp;
indices1=(bool*)calloc(dim_x,sizeof(bool));
for (i=0; i<dim_x; i++) 
	if (tmp[i]!='0') indices1[i]=true;

cout << "Entrez la matrice des indices d'ordre 2 a calculer" << endl;
indices2=(bool*)calloc(dim_x*dim_x,sizeof(bool));
for (i=0; i<dim_x; i++) {
	cin >> tmp;
	for (j=0; j<dim_x; j++) 
		if (tmp[j]!='0') indices2[i*dim_x+j]=true;
}

if (choix=='s') {

	cout << "Entrez le vecteur des indices totaux a calculer" << endl;
	cin >> tmp;
	indicest=(bool*)calloc(dim_x,sizeof(bool));
	for (i=0; i<dim_x; i++) 
		if (tmp[i]!='0') indicest[i]=true;
}

cout << "Indices calcules en pourcentages? 000=non ordre 1,2 et totaux" << endl;
cin >> tmp;
rel=(bool*)calloc(3,sizeof(bool));
for (i=0; i<3; i++) 
	if (tmp[i]!='0') rel[i]=true;

if (choix!='s') { 

	cout << "Echantillonnage deterministe? (0 pour non)" << endl;
	cin >> tmp;
	if (tmp[0]!='0') deterministe=true;
}	
	
cout << "Affichage des resultats intermediaires? (0 pour non)" << endl;
cin >> tmp;
if (tmp[0]!='0') affichage=true;

if (choix=='s') sobol(getFunc(name),dim_x,S,R,lois,paramlois,logx,logy[0],
			indices1,indices2,indicest,rel,affichage);
	
else if (choix=='e') entrop(getFunc(name),dim_x,dim_y,S,n,R,lois,paramlois,logx,logy,
			indices1,indices2,rel,deterministe,affichage);   
		
else if (choix=='m') mix(getFunc(name),dim_x,S,n,R,lois,paramlois,logx,logy[0],
			indices1,indices2,deterministe,affichage);

return 0;

}
