#include "sample.h"

#include <iostream>

/* fonctions d'echantillonnage (sans correlation modulo celles du generateur aleatoire): */

std::vector<double> sampling(int dim_x,long S,int* lois,double* paramlois,int z,bool deterministe) {

bool stop;
int i;
long j,eS=(long)floor(pow(S,(double)1/dim_x));
double val,valp,d=0,norm,normp,pS=pow(S,(double)1/dim_x);
double* pas=NULL;
std::vector<double> res(dim_x*S);

/* initialisation du generateur aleatoire */
srand(z);

/* on complete les bornes des lois si necessaire */
for (i=dim_x-1; i>=0; i--)
	if (lois[i]==0 && paramlois[i*4]>=paramlois[i*4+1]) paramlois[i*4+1]=1;
	else if (lois[i]==1 && paramlois[i*4+1]<=0) paramlois[i*4+1]=1;
	else if (lois[i]==2 && paramlois[i*4+1]<=0) paramlois[i*4+1]=1;
	else if (lois[i]==6 && paramlois[i*4]>=paramlois[i*4+2]) {
		paramlois[i*4+1]=0.5;
		paramlois[i*4+2]=1;
	}
	else if (lois[i]==7 && paramlois[i*4]>=paramlois[i*4+3]) {
		paramlois[i*4+1]=0.25;
		paramlois[i*4+2]=0.75;
		paramlois[i*4+3]=1;
	}
	
/* simulations des lois */
if (deterministe) { 
	stop=false;
	pas=(double*)calloc(dim_x,sizeof(double));
	for (i=dim_x-1; i>=0; i--) pas[i]=(paramlois[i*4+1]-paramlois[i*4])/pS;
	// initialisation du resultat
	res[dim_x*S-1]=-1;
	for (i=dim_x-2; i>=0; i--) res[(i+1)*S-1]=0;
	for (i=dim_x-1; i>=0; i--)
		for (j=S-2; j>=0; j--) res[i*S+j]=0;

	for (j=S-1; j>=0; j--) {
		res[(dim_x-1)*S+j]++;
		for (i=dim_x-1; i>=0; i--) {
			if (res[i*S+j]>=eS) {
				if (i<=0) {
					stop=true;
					j++;
					break;
				}
				else {
					res[i*S+j]=0;
					res[(i-1)*S+j]++;
				}
			}
			else break;
		}
		if (stop) break;

		// recopie sur l'etage suivant pour propager l'addition (+ normalisation)
		if (j>0)
			for (i=dim_x-1; i>=0; i--) {
				res[i*S+j-1]=res[i*S+j]++;
				res[i*S+j]=paramlois[i*4]+res[i*S+j]*pas[i];
			}
		else {
			// normalisation des valeurs
			for (i=dim_x-1; i>=0; i--) res[i*S+j]=paramlois[i*4]+(res[i*S+j]+1)*pas[i];
		}
	}
	if (j>0) {
		// completion avec des valeurs aleatoires (en nombre negligeable, nul si pS est entier)
		while (--j>=0)
			for (i=dim_x-1; i>=0; i--) res[i*S+j]=paramlois[i*4]+(paramlois[i*4+1]-paramlois[i*4])*(double)rand()/RAND_MAX;
	}
	
	return res;
}

for (i=dim_x-1; i>=0; i--) { 
	if (lois[i]==0) { /* loi uniforme */
		for (j=S-1; j>=0; j--)
			res[i*S+j]=(paramlois[i*4+1]-paramlois[i*4])*(double)rand()/RAND_MAX+paramlois[i*4];
	}
	else if (lois[i]==1) { /* loi normale */
		j=S-1;
		while (j>=0) {
			val=tan(0.5*M_PI*(1-2*(double)rand()/RAND_MAX));
			d=exp(-0.5*(val-paramlois[i*4])*(val-paramlois[i*4]))/(paramlois[i*4+1]*sqrt(2*M_PI));
			if ((double)rand()/RAND_MAX<=d*paramlois[i*4+1]*sqrt(2*M_PI)) {
				res[i*S+j]=d;
				j--;
			}
		}
	}
	else if (lois[i]==2) { /* loi lognormale */
		j=S-1;
		while (j>=0) {
			val=tan(0.5*M_PI*(double)rand()/RAND_MAX);
			d=exp(-0.5*(log(val)-paramlois[i*4])*(log(val)-paramlois[i*4]))/(val*paramlois[i*4+1]*sqrt(2*M_PI));
			if ((double)rand()/RAND_MAX<=d*paramlois[i*4+1]*sqrt(2*M_PI)*exp(paramlois[i*4]-1)/sqrt(M_E)) {
				res[i*S+j]=d;
				j--;
			}
		}
	} 
	else if (lois[i]==3) { /* loi weibull */    
		for (j=S-1; j>=0; j--) res[i*S+j]=pow((log(1-(double)rand()/RAND_MAX))/(-paramlois[i*4]),1/paramlois[i*4+1]);
	}
	else if (lois[i]==4) { /* loi exponentielle */
		for (j=S-1; j>=0; j--) res[i*S+j]=(log(1-(double)rand()/RAND_MAX))/(-paramlois[i*4]);
	}
	else if (lois[i]==5) { /* loi beta */ 
		j=S-1;
		while (j>=0) { 
			val=pow((double)rand()/RAND_MAX,1/paramlois[i*4]);
			valp=pow((double)rand()/RAND_MAX,1/paramlois[i*4+1]);
			if (val+valp<=1) { 
				res[i*S+j]=val/(val+valp); 
				j--;
			}
		}
	}
	else if (lois[i]==6) { /* loi triangulaire */
		norm=1/(paramlois[i*4+1]-paramlois[i*4]);
		normp=1/(paramlois[i*4+2]-paramlois[i*4+1]);
		j=S-1;
		while (j>=0) { 
			val=(paramlois[i*4+1]-paramlois[i*4])*(double)rand()/RAND_MAX+paramlois[i*4];
			valp=(double)rand()/RAND_MAX;
			if ((val<=paramlois[i*4+1] && valp<=norm*(val-paramlois[i*4])) 
				|| (val>paramlois[i*4+1] && valp<=normp*(paramlois[i*4+2]-val))) {
				res[i*S+j]=val; 
				j--;
			}
		}
	} 
	else if (lois[i]==7) { /* loi trapezoidale */
		norm=1/(paramlois[i*4+1]-paramlois[i*4]);
		normp=1/(paramlois[i*4+3]-paramlois[i*4+2]);
		j=S-1;
		while (j>=0) {
			val=(paramlois[i*4+1]-paramlois[i*4])*(double)rand()/RAND_MAX+paramlois[i*4];
			valp=(double)rand()/RAND_MAX;
			if (val<=paramlois[i*4+1]) {
				if (valp<=norm*(val-paramlois[i*4])) {
					res[i*S+j]=val;
					j--;
				}
			}
			else if (val>=paramlois[i*4+2]) {
				if (valp<=normp*(paramlois[i*4+3]-val)) {
					res[i*S+j]=val;
					j--;
				}
			}
			else {
				res[i*S+j]=val;
				j--;
			}
		}
	}
}

return res;

}

