samplingDet<-function(dim_x,S=10000,lois=rep(0,dim_x),
                   paramlois=array(0,dim=c(4,dim_x))) {

#---------------------------------------------------------------------------
#Tirage deterministe de N jeux de parametres                               
#---------------------------------------------------------------------------
#Arguments obligatoires :                                                   
#               dim_x : nombre de parametres d'entree dans le modele        
#                                                                           
#Arguments facultatifs :                                                    
#             S : nombre de jeux de simulations                                           
#             paramlois : tableau contenant les 2 parametres (min,max) 
#               pour chaque Vi (range par colonne). Par defaut, on a 0                                          
#---------------------------------------------------------------------------
#Sortie : la matrice des N simulations des dim_x parametres                
#---------------------------------------------------------------------------

eS<-floor(S^(1/dim_x))
pS<-S^(1/dim_x)
res<-matrix(nrow=S,ncol=dim_x)

# on complete les bornes des lois si necessaire 
for (i in 1:dim_x) 
	if (lois[i]==0 && paramlois[1,i]>=paramlois[2,i]) paramlois[2,i]<-1
	else if ((lois[i]==1 || lois[i]==2) && paramlois[2,i]<=0) paramlois[2,i]<-1
	else if (lois[i]==6 && paramlois[1,i]>=paramlois[3,i]) {
		paramlois[2,i]<-0.5
		paramlois[3,i]<-1
	}
	else if (lois[i]==7 && paramlois[1,i]>=paramlois[4,i]) {
		paramlois[2,i]=0.25
		paramlois[3,i]=0.75
		paramlois[4,i]=1
	}

# simulations de lois uniformes
for (i in 1:dim_x) { 
	stop<-FALSE
	pas<-(paramlois[2,i]-paramlois[1,i])/pS
	# initialisation du resultat
	res<-matrix(0,nrow=S,ncol=dim_x)
		res[1,1]<- -1
		mem<-0		  
 
	for (j in 1:S) 
		if (!stop) {
			res[j,1]<-res[j,1]+1
			for (k in 1:dim_x) 
			if (res[j,k]>=eS)
				if (k<dim_x) {
					res[j,k]<-0  
					res[j,k+1]<-res[j,k+1]+1
				}
				else stop<-TRUE
            
			# recopie sur l'etage suivant pour propager l'addition (+ normalisation)
			if (!stop && j<S) {
				res[j+1,]<-res[j,]
				res[j,]<-paramlois[1,i]+(res[j,]+0.5)*pas
				res[j+1,1]<-res[j+1,1]+1
			}
			else 
				# normalisation des valeurs
				res[j,]<-paramlois[1,i]+(res[j,]+0.5)*pas
		}
		else break
	if (j<S) 
	# completion avec des valeurs aleatoires (en nombre negligeable, nul si pS est entier)
	while (j<=S) {
		res[j,]<-runif(dim_x,paramlois[1,i],paramlois[2,i])
		j<-j+1
	}                
}

return(res)
}




