samplingSimple<-function(dim_x,N=10000,lois=rep(0,dim_x),
                   paramlois=array(0,dim=c(4,dim_x))) {

#---------------------------------------------------------------------------
#Tirage aleatoire simple de N jeux de parametres                            
#---------------------------------------------------------------------------
#Arguments obligatoires :                                                   
#               dim_x : nombre de parametres d'entree dans le modele        
#                                                                           
#Arguments facultatifs :                                                    
#             nom : vecteur des noms des parametres simules                 
#               (defini comme : c("V1","V2","V3",...))                     
#             N : nombre de jeux de simulations                             
#             lois : vecteur contenant les types de distribution de proba   
#               pour chaque Vi                                              
#               0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull           
#               Par defaut, on prend la loi uniforme                        
#             paramlois : tableau contenant les 2 parametres de chaque loi  
#               pour chaque Vi (range par colonne) :                        
#               (min,max) pour uniforme, (moy,ecart-type) pour loi normale, 
#               (moy du log, ecart-type du log) pour loi lognormale,        
#               (forme,echelle) pour loi de Weibull                         
#               Par defaut, on a 0                                          
#---------------------------------------------------------------------------
#Sortie : la matrice des N simulations des dim_x parametres                 
#---------------------------------------------------------------------------
  
x<-matrix(0,nrow=N,ncol=dim_x)

#on complete les bornes des lois uniformes
for (i in 1:dim_x) 
	if (lois[i]==0 && paramlois[1,i]>=paramlois[2,i]) paramlois[2,i]<-1
	else if (lois[i]==6 && paramlois[1,i]>=paramlois[3,i]) {
		paramlois[2,i]<-0.5
		paramlois[3,i]<-1
	}
	else if (lois[i]==7 && paramlois[1,i]>=paramlois[4,i]) {
		paramlois[2,i]=0.25
		paramlois[3,i]=0.75
		paramlois[4,i]=1
	}
	
for(i in 1:dim_x) {
	if (lois[i]==0) # loi uniforme
		x[,i]<-runif(N,min=paramlois[1,i],max=paramlois[2,i])
	else if (lois[i]==1) # loi normale
		x[,i]<-rnorm(N,mean=paramlois[1,i],sd=paramlois[2,i])
	else if (lois[i]==2) # loi lognormale
		x[,i]<-rlnorm(N,meanlog=paramlois[1,i],sdlog=paramlois[2,i])
	else if (lois[i]==3) # loi weibull
		x[,i]<-rweibull(N,shape=paramlois[1,i],scale=paramlois[2,i])
	else if (lois[i]==4) # loi exponentielle
		x[,i]<-rexp(N,rate=paramlois[1,i])
	else if (lois[i]==5) # loi beta
		x[,i]<-rbeta(P,shape1=paramlois[1,i],shape2=paramlois[2,i])
	else if (lois[i]==6) { # loi triangulaire
		norm<-1/(paramlois[2,i]-paramlois[1,i])
		normp<-1/(paramlois[3,i]-paramlois[2,i])
		j<-1
		while (j<=N) {
			aux<-runif(1,min=paramlois[1,i],max=paramlois[3,i])
			auxp<-runif(1)
			if ((aux<=paramlois[2,i] && auxp<=norm*(aux-paramlois[1,i])) || (aux>paramlois[2,i] && auxp<=normp*(paramlois[3,i]-aux))) {
				x[j,i]<-aux
				j<-j+1
			}
		}
	}
	else if (lois[i]==7) { # loi trapezoidale
		norm<-1/(paramlois[2,i]-paramlois[1,i])
		normp<-1/(paramlois[4,i]-paramlois[3,i])
		j<-1
		while (j<=N) {
			aux<-runif(1,min=paramlois[1,i],max=paramlois[3,i])
			auxp<-runif(1)
			if (aux<=paramlois[2,i]) {
				if (auxp<=norm*(aux-paramlois[1,i])) {
					x[j,i]<-aux
					j<-j+1
				}
			}
			else if (aux>=paramlois[3,i]) {
				if (auxp<=normp*(paramlois[4,i]-aux)) {
					x[j,i]<-aux
					j<-j+1
				}
			}
			else {
				x[j,i]<-aux
				j<-j+1
			}
		}
	}
}

return(x)
}
